#!/usr/bin/env python3.7

import sys
import re
import string
import unittest


white_space     = re.compile('\s+')
trail_space     = re.compile('\s*$')
arg_term        = re.compile('\s|$')
full_comment    = re.compile('^#')
dash            = re.compile('-')
csv             = re.compile('.csv');
slash           = re.compile('/')
match_heuristic = re.compile("-H'?\([^#]*\)'?")
match_class     = re.compile('CLASS_[a-zA-Z-0-9]*$')
eval_f_sep      = re.compile('\),')
problem_ext     = re.compile('\.[a-z]*$')



def extract_arg(line, mopt):
    l = line[mopt.end():]
    m = arg_term.search(l)
    res = l[0:m.start()]

    if res == "":
        # raise RuntimeError, "Argument to option in command line missing: "+line[mopt.start():]
        pass

    return res

def extract_opt_arg(line, mopt, opt):
    l = line[mopt.end():]
    # print "extract_opt_arg(", line, ") l=", l
    if len(l)==0:
        return opt
    if l[0]!="=":
        return opt

    m = arg_term.search(l[1:])
    res = l[1:m.start()+1]
    #print "extract_opt_arg(",line, opt, ") =>", res
    return res

def clean_heuristics(str):
    str = re.sub(white_space, "", str); 
    res = re.sub("\*", ".", str);
    return res
    


match_ac_l      = re.compile(" --ac-handling=")
match_acnag_l   = re.compile(" --ac-non-aggressive")
match_litsel_s  = re.compile(" -W *")
match_litsel_l  = re.compile(" --literal-selection-strategy=")
match_op_l      = re.compile(" --select-on-processing-only")
match_ipl_s     = re.compile(" -i")
match_ipl_l     = re.compile(" --inherit-paramod-literals")
match_ipg_s     = re.compile(" -j")
match_ipg_l     = re.compile(" --inherit-goal-pm-literals")
match_ipc_l     = re.compile(" --inherit-conjecture-pm-literals")
match_pic_l     = re.compile(" --prefer-initial-clauses")
match_split_l   = re.compile(" --split-clauses=")
match_splitm_l  = re.compile(" --split-method=")
match_splita_l  = re.compile(" --split-aggressive")
match_splitr_l  = re.compile(" --split-reuse-defs")
match_der_l     = re.compile(" --destructive-er")
match_sder_l    = re.compile(" --strong-destructive-er")
match_dera_l    = re.compile(" --destructive-er-aggressive")
match_demod_s   = re.compile(" -F *")
match_demod_l   = re.compile(" --forward_demod_level=")
match_snm_l     = re.compile(" --selection-neg-min=")
match_g_demod_s = re.compile(" --prefer-general-demodulators")
match_g_demod_l = re.compile(" -g")
match_unproc_s  = re.compile(" --simplify-with-unprocessed-units=")
match_unproc_sd = re.compile(" --simplify-with-unprocessed-units")
match_fcsr      = re.compile(" --forward-context-sr")
match_fcsra     = re.compile(" --forward-context-sr-aggressive")
match_bcsr      = re.compile(" --backward-context-sr")
match_simul_pm  = re.compile(" --simul-paramod");
match_osimul_pm = re.compile(" --oriented-simul-paramod");
match_presat_ir = re.compile(" --presat-simplify");
match_sos_types = re.compile(" --sos-uses-input-types");
match_condense  = re.compile(" --condense");
match_condag    = re.compile(" --condense-aggressive");

match_sat_proc_int    = re.compile(" --satcheck-proc-interval")
match_sat_gen_int     = re.compile(" --satcheck-gen-interval")
match_sat_tt_int      = re.compile(" --satcheck-ttinsert-interval")
match_satcheck        = re.compile(" --satcheck(?=[^-])")
match_sat_norm_const  = re.compile(" --satcheck-normalize-const")
match_sat_norm_unproc = re.compile(" --satcheck-normalize-unproc")
match_sat_dec_limit   = re.compile(" --satcheck-decision-limit")



class EOption(object):
    def __init__(self, longopt, shortopt, argtype=None, optdef=None):
        """
        argtype in None, "heuristic", "string", "uint", "int"
        """
        self.longopt  = longopt
        self.shortopt = shortopt
        self.argtype  = argtype
        self.optdef   = optdef

    def hasOptArg(self):
        return self.optdef!=None

    def hasReqArg(self):
        return self.argtype!=None and not self.hasOptArg()

    def parseShort(self, optstr):
        mo = re.search(self.shortopt+" *", optstr)
        if not mo:
            if self.argtype == None:
                return False
            else:
                return None
        elif not self.argtype:
            return True

        res = extract_arg(optstr, mo)
        return res

    def parseLong(self, optstr):
        mo = re.search(self.longopt, optstr)
        if not mo:
            if self.argtype == None:
                return False
            else:
                return None
        elif not self.argtype:
            return True
            
        res = extract_opt_arg(optstr, mo, self.optdef)
        return res


    def parse(self, optstr):
        res = None
        if self.shortopt:
            res = self.parseShort(optstr)
        if res==None:
            if self.longopt:
                res = self.parseLong(optstr)

        if self.argtype=="heuristic":
            res = clean_heuristics(res)
        return res





class TestOpts(unittest.TestCase):
    """
    Test option functionality.
    """

    def setUp(self):
        self.teststr = "# eprover --delete-bad-limit=2000000000 --definitional-cnf=24 -s --print-statistics -R --print-version --free-numbers --simul-paramod --forward-context-sr --strong-destructive-er --destructive-er-aggressive --destructive-er --presat-simplify -tKBO6 -winvfreqrank -c1 -Ginvfreq -F1 -WSelectMaxLComplexAvoidPosPred --satcheck-proc-interval=5000 --satcheck=ConjMinMinFreq -H(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2)) --cpu-limit=300 /export/starexec/sandbox/benchmark/theBenchmark.p"

        self.opt1 = EOption("--define-heuristic","-H", "heuristic");
        self.opt2 = EOption("--satcheck", None, "string", "FirstConst");
        self.opt3 = EOption("--destructive-er", None, None);
        self.opt4 = EOption("--select-on-processing-only", None, None);


    def testParse(self):
        print(self.opt1.parse(self.teststr))
        print(self.opt2.parse(self.teststr))
        print(self.opt3.parse(self.teststr))
        print(self.opt4.parse(self.teststr))

if __name__ == '__main__':
    unittest.main()
