#!/usr/bin/env python3

"""
runscripts.py

Usage: runscripts.py

Generate all possible LPO/KBO variant runscripts (with/without -c1)
from a built-in template.

Options:

-h Print this help.

Copyright 2019 Stephan Schulz, schulz@eprover.org

This code is part of the support structure for the equational
theorem prover E. Visit

 http://www.eprover.org

for more information.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program ; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston,
MA  02111-1307 USA

The original copyright holder can be contacted as

Stephan Schulz
DHBW Stuttgart
Informatik
Jaegerstrasse 56
70174 Stuttgart
Germany

or (preferably) via email (address above).
"""

import sys
import re
import string
import os
import os.path
import getopt
from pylib_heu_enc import *

template = """#!/usr/bin/tcsh
echo -n "%% Problem    : " ; head -2 $1 | tail -1 | sed -e "s/.*  : //"
set ProblemSPC=`grep " SPC " $1 | sed -e "s/.* : //"`
set defaultargs="--delete-bad-limit=2000000000 --definitional-cnf=24 -s --print-statistics -R --print-version --free-numbers"
set flags=" --simul-paramod --forward-context-sr --destructive-er-aggressive --destructive-er -F1 -WNoSelection --presat-simplify --strong-rw-inst "
set ordering = "-t%s -G%s -w%s %s "
set heuristic=" %s "
set final="--cpu-limit="$STAREXEC_CPU_LIMIT" "$1
set ecmd="./eprover ""$defaultargs $flags $ordering $heuristic $final"

if ( `expr "$ProblemSPC" : "FOF.*"` || `expr "$ProblemSPC" : "CNF.*"` ) then
	echo "%% Command    : " $ecmd
	/home/starexec/bin/GetComputerInfo -p THIS Model CPUModel RAMPerCPU OS | \
        sed -e "s/Computer     /%% Computer   /" \
            -e "s/Model        /%% Model      /" \
            -e "s/CPUModel     /%% CPU        /" \
            -e "s/RAMPerCPU    /%% Memory     /" \
            -e "s/OS           /%% OS         /"
	echo -n "%% CPULimit   : " ; echo "$STAREXEC_CPU_LIMIT"
	echo -n "%% DateTime   : " ; date
	echo "%% CPUTime    : "
	$ecmd
else
    echo "%% SZS status Inappropriate"
endif
"""


"""
Name template variables:

 0: H%4s  - 4 character (base 36) encoding of running number of clause
            selection heuristic
 1: %1s  - [L]PO4 or [K]BO6
 2: %02d  - Numeric encoding of precedence generation scheme
 3: %02d  - Numeric encoding of weight generation scheme
 4: %2s   - cn or cd, where d is a digit and the weight for constants
 5: C%1s  - Literal comparison [n]ormal, transfinite EqM[a]x or EqM[i]n
 6: F%1d  - Forward rewrite level
 7: G%1s  - [s]trong instances or [n]ormal rewriting
 8: D%1s  - Destructive equality resolution: [n]one, [e], or [a]gressive
 9: C%1s  - Forward Context-SR: [N]none, [F]orward, [A]gressive,
10: B%1s  - Backward Context-SR: [N]one, [B]ackward
11: S%1s  - Superposition, [N]ormal, [S]imultaneous, ...
12: P%1s  - Presimplification, - or [Y]es
13: p%1s  - Preprocessing: [D]efinitions, [P]lain, [N]one
14: L%3s  - Literal selection numerical encoding (base 36)
"""

name_template = "starexec_run_E---2.0_G_H%4s_%1s_%02d_%02d_%2s_C%s1_F%1d_G%1s_D%1s_C%s_B%1s_S%1s_P%1s_p%1s_L%3s"


default_clausesel_heuristic = clause_heuristics[0]

default_fillers=[
    make_henc(clausesel_heuristics["G_E___207_C01_F1_SE_CS_SP_PI_S5PRR_S0Y"]),
    "K",
    precs.index("invfreq"),
    weights.index("invfreqrank"),
    "cn",
    "n",
    1,
    "n",
    "a",
    "






for i in range(len(precs)):
    name = name_templates[tmpsel]%("B", i, 0)
    if not "type" in precs[i]:
        script = templates[tmpsel]%("LPO4", precs[i], weights[0])
        fp = open(name, "w");
        fp.write(script)
        fp.close()


for i in range(len(precs)):
    for j in  range(len(weights)):
        if not "type" in precs[i] and not "type" in weights[j]:
            name = name_templates[tmpsel]%("C", i, j)
            script = templates[tmpsel]%("KBO6", precs[i], weights[j])
            fp = open(name, "w");
            fp.write(script)
            fp.close()
            os.chmod(name, 0o755)




q
