#!/usr/bin/env python3
"""
tptp_perf_analyse.py

Usage: tptp_perf_analyse.py <db-file> [-s <system>] *

Parse a TPTP results database file ("ProblemAndSolutionStatistics")
and extract E-toolchain compatible files for the specified
systems.

Options:
 -h
--help      Print this information.

 -s <system>
--select <system>

  Extract data for the specified system.

Copyright 2020 Stephan Schulz, schulz@eprover.org

This code is part of the support structure for the equational
theorem prover E. Visit

 http://www.eprover.org

for more information.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program ; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston,
MA  02111-1307 USA

The original copyright holder can be contacted via email or as

Stephan Schulz
DHBW Stuttgart
Fakultaet Technik
Informatik
Rotebuehlplatz 41
70178 Stuttgart
Germany
"""


import sys
import re
import getopt

selectors = []



def processOptions(optlist):
    """
    Process options.
    """
    global selectors

    for opt, arg in optlist:
        # print(opt,arg)
        if opt == "-h" or opt == "--help":
            print(__doc__)
            sys.exit()
        elif opt == "-s" or opt == "--select":
            selectors.append(arg)


transStatus = {
    "CSA":"N",
    "FTH":"F",
    "FUN":"F",
    "GUP":"F",
    "IAP":"F",
    "SAT":"N",
    "THM":"T",
    "TMO":"F",
    "UNK":"F",
    "UNS":"T",
    }

transReason = {
    "CSA":"success",
    "FTH":"unknown",
    "FUN":"unknown",
    "GUP":"incomplete",
    "IAP":"incomplete",
    "SAT":"success",
    "THM":"success",
    "TMO":"maxtime",
    "UNK":"unknown",
    "UNS":"success",
    }


def transRecord(record):
    (name, status, time, witness) = record
    return "%-29s %s %8.3f %s\n"%(name+".p", transStatus[status], float(time), transReason[status])

class ResStore(object):
    def __init__(self, selectors):
        self.selectors = selectors
        self.store = {}
        for i in selectors:
            self.store[i] = []

    def isRelevant(self, selector):
        return selector in self.selectors

    def add(self, system, record):
        self.store[system].append(record)

    def resStr(self, selector):
        return "# "+selector+"\n"+ \
            "".join(map(transRecord, self.store[selector]))


def processEntry(entry, store):
    lines = entry.split("\n")
    name = lines[0].split()[0]

    for line in lines[1:-1]:
        system, status, time, witness = line.split()[0:4]
        if store.isRelevant(system):
            store.add(system, (name, status, time, witness))


def readEntry(dbfile):
    res = []
    line = dbfile.readline()
    while line and line !="\n":
        if line[0]!="%":
            res.append(line)
        line = dbfile.readline()

    return "".join(res)


if __name__ == '__main__':
    print("#", " ".join(sys.argv))

    opts, args = getopt.gnu_getopt(sys.argv[1:], "hs:", ["help",
    "--select="])

    processOptions(opts)

    if len(args)!=1:
        print(args)
        print(__doc__)
        sys.exit()

    store = ResStore(selectors)

    with open(args[0], "r") as dbfile:
        entry = readEntry(dbfile)
        while entry:
            processEntry(entry, store)
            entry = readEntry(dbfile)

    for i in selectors:
        print(store.resStr(i),end='')
