#!/usr/bin/env python3

"""
pylib_heu_enc.py

Text encoding of a large number of successful clause selection
heuristics in standardised form. Also provides names of weight and
literal selection  functions, and helper functions
for encoding heuristic choices into strings.

Copyright 2020 Stephan Schulz, schulz@eprover.org

This code is part of the support structure for the equational
theorem prover E. Visit

 http://www.eprover.org

for more information.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program ; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston,
MA  02111-1307 USA

The original copyright holder can be contacted as

Stephan Schulz
DHBW Stuttgart
Informatik
Jaegerstrasse 56
70174 Stuttgart
Germany

or (preferably) via email (address above).
"""

import unittest

clause_heuristics = {
    "C07_19_nc_SAT001_MinMin_rr": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"C12_02_nc_SAT001_MinMin_rr": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
    "G_____X1276__C12_02_nc_F1_AE_CS_SP_S5PRR_S2S": "(20.Diversityweight(ConstPrio,1,1,1.5,1,1,-1.000000,0.000000,-1.000000,2.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_E___001_K07_F1_PI_AE_O1": "(3.Refinedweight(PreferGoals,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,0.5),1.Clauseweight(ConstPrio,1,1,1.5),1.FIFOWeight(ConstPrio))",
"G_E___001_K07_F2_PI_AE_S1": "(3.Refinedweight(PreferGoals,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,0.5),1.Clauseweight(ConstPrio,1,1,1.5),1.FIFOWeight(ConstPrio))",
"G_E___001_K07_F2_PI_AE_Z1": "(3.Refinedweight(PreferGoals,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,0.5),1.Clauseweight(ConstPrio,1,1,1.5),1.FIFOWeight(ConstPrio))",
"G_E___001_K16_F1_PI_AE_Y1": "(3.Refinedweight(PreferGoals,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,0.5),1.Clauseweight(ConstPrio,1,1,1.5),1.FIFOWeight(ConstPrio))",
"G_E___001_K16_F2_PI_AE_Y1": "(3.Refinedweight(PreferGoals,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,0.5),1.Clauseweight(ConstPrio,1,1,1.5),1.FIFOWeight(ConstPrio))",
"G_E___001_K18_F1_PI_AE_S4_CS_SP_S2S": "(10.ConjectureSymbolWeight(ConstPrio,10,10,5,5,5,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___001_K18_F1_PI_AE_Z1": "(3.Refinedweight(PreferGoals,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,0.5),1.Clauseweight(ConstPrio,1,1,1.5),1.FIFOWeight(ConstPrio))",
"G_E___003_K18_F1_AE_S4_CS_SP_S0Y": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.2,100,100,100,100,1.5,1.5,1),1.FIFOWeight(ConstPrio))",
"G_E___003_K18_F1_AE_S4_CS_SP_S2S": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.2,100,100,100,100,1.5,1.5,1),1.FIFOWeight(ConstPrio))",
"G_E___003_K18_F1_PI_AE_S4_CS_OS_S0Y_backup": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.2,100,100,100,100,1.5,1.5,1),1.FIFOWeight(ConstPrio))",
"G_E___003_K18_F1_PI_AE_S4_CS_SP_S0Y": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.2,100,100,100,100,1.5,1.5,1),1.FIFOWeight(ConstPrio))",
"G_E___003_K18_F1_PI_AE_S4_CS_SP_S2S": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.2,100,100,100,100,1.5,1.5,1),1.FIFOWeight(ConstPrio))",
"G_E___006_C18_F1_PI_AE_Q4_CS_SP_S2S": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.5,100,100,100,50,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___006_K18_F1_PI_AE_S4_CS_SP_S0Y": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.5,100,100,100,50,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___007_K18_F1_PI_AE_S4_CS_SP_S0Y": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.2,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___008_C18_F1_AE_Q4_CS_SP_S2S": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___008_C18_F1_PI_AE_CS_SP_CO_S4S": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___008_C18_F1_PI_SE_CS_SP_CO_S4S": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___008_C45_F1_PI_AE_Q4_CS_SP_S4c": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___008_C45_F1_PI_AE_Q4_CS_SP_S4d": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___008_C45_F1_PI_S5PRR_SE_Q4_CS_SP_S4S": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___008_C45_F1_PI_SE_Q4_CS_SP_S0Y": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___008_C45_F1_PI_SE_Q4_CS_SP_S4SA": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___008_C45_F1_PI_SE_Q4_CS_SP_S4SI": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___008_K18_F1_PI_AE_S4_CS_SP_S0Y": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___010_B02_F1_PI_AE_S4_CS_SP_S0Y": "(4.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),3.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___010_K07_F1_PI_AE_S4_CS_SP_S0Y": "(4.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),3.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___010_K18_F1_PI_AE_CS_SP_S0Y": "(4.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),3.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___010_K18_F1_PI_AE_S4_CS_SP_S0Y_backup": "(4.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),3.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___010_K18_F1_PI_AE_S4_CS_SP_S2S": "(4.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),3.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___010_K18_F1_PI_AE_S4_CS_SP_S2S_backup": "(4.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),3.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___010_K18_F1_PI_AE_S4_CS_SP_S3S": "(4.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),3.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___010_K18_F2_PI_AE_S4_CS_SP_S0Y": "(4.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),3.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___011_K18_F1_PI_AE_S4_CS_SP_S0Y": "(7.ConjectureRelativeSymbolWeight(ConstPrio,0.5,100,100,100,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___011_K18_F1_PI_AE_S4_CS_SP_S0Y_backup": "(7.ConjectureRelativeSymbolWeight(ConstPrio,0.5,100,100,100,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___012_C18_F1_PI_AE_Q4_CS_SP_PS_S0Y": "(7.ConjectureRelativeSymbolWeight(ConstPrio,0.5,100,100,100,100,1.5,1.5,1),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___012_K18_F1_PI_AE_S4_CS_SP_S0Y": "(7.ConjectureRelativeSymbolWeight(ConstPrio,0.5,100,100,100,100,1.5,1.5,1),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___024_B31_F1_PI_AE_Q4_CS_SP_S2S": "(4.ConjectureGeneralSymbolWeight(SimulateSOS,100,100,100,50,50,50,50,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,100,100,100,50,50,1000,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___030_K01_F1_PI_AE_Y": "(2.Clauseweight(PreferGoals,1,1,1),3.Refinedweight(PreferNonGoals,1,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_E___030_K01_F2_PI_AE_Z": "(2.Clauseweight(PreferGoals,1,1,1),3.Refinedweight(PreferNonGoals,1,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_E___040_K16_F2_PI_S1": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(ConstPrio,1,1,2),1.FIFOWeight(ConstPrio))",
"G_E___040_K18_F1_O1": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(ConstPrio,1,1,2),1.FIFOWeight(ConstPrio))",
"G_E___040_K18_F1_PI_AE_S4_AS_O1": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(ConstPrio,1,1,2),1.FIFOWeight(ConstPrio))",
"G_E___040_K18_F1_PI_AE_S4_O1_csr": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(ConstPrio,1,1,2),1.FIFOWeight(ConstPrio))",
"G_E___040_K18_F1_PI_AE_a0": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(ConstPrio,1,1,2),1.FIFOWeight(ConstPrio))",
"G_E___040_K18_F1_a0": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(ConstPrio,1,1,2),1.FIFOWeight(ConstPrio))",
"G_E___041_C18_F1_PI_AE_Q4_CS_SP_S0Y": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),3.ConjectureRelativeSymbolWeight(ConstPrio,0.3,100,100,100,100,2.5,1,1),1.FIFOWeight(ConstPrio))",
"G_E___042_C18_F1_PI_AE_Q4_CS_SP_PS_S0Y": "(20.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.FIFOWeight(ConstPrio))",
"G_E___042_C45_F1_PI_AE_Q4_CS_SP_PS_S4S": "(20.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.FIFOWeight(ConstPrio))",
"G_E___044_K18_F1_PI_AE_S4_CS_OS_S1S": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),2.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___044_K18_F1_PI_AE_S4_CS_SP_S2S": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),2.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___044_K18_F1_PI_AE_S4_O1_csr": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),2.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___044_K18_F1_PI_AE_S4_S1": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),2.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___044_K18_F1_PI_AE_S4_S1S": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),2.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___044_K18_F1_PI_AE_S4_SP_S2S": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),2.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___045_B07_F1_PI_AE_S4_CS_SP_S0Y": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___045_K07_F1_PI_AE_S4_CS_Y": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___045_K10_F1_PI_AE_S4_CS_Y": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___045_K18_F1_PI_AE_CS_SP_S00": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___045_K18_F1_PI_AE_S4_CS_I": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___045_K18_F1_PI_AE_S4_CS_SP_S0Y": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___045_K18_F1_PI_AE_S4_CS_SP_S2S": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___045_K18_F1_PI_AE_S4_CS_W": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___045_K18_F1_PI_AE_S4_CS_Y": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___045_K18_F1_PI_AE_S4_O1_csr": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___045_K18_F1_PI_AE_S4_SP_S0Y": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___045_K18_F2_PI_AE_S4_CS_Y": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___045_K19_F1_PI_AE_S4_CS_Y": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___046_K18_F1_PI_AE_S4_CS_S1": "(8.Refinedweight(SimulateSOS,1,1,2,1.5,2),6.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),4.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___047_K18_F1_PI_AE_S4_CS_J": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),2.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___047_K18_F1_PI_AE_S4_CS_O1": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),2.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___047_K18_F1_PI_AE_S4_CS_Y": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),2.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___047_K18_F1_PI_AE_S4_O1": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),2.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___047_K18_F1_PI_AE_S4_Y1": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),2.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___049_K18_F1_PI_AE_S4_CS_Y": "(4.Refinedweight(SimulateSOS,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,1.5),2.Clauseweight(PreferProcessed,1,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"G_E___051_K18_F1_PI_AE_S4_S1": "(10.Refinedweight(SimulateSOS,1,2,2,2,0.5),10.Refinedweight(DeferSOS,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"G_E___060_C18_F1_PI_AE_CS_SP_S0Y": "(1.RelevanceLevelWeight(ConstPrio,2,2,0,2,100,100,100,100,1.5,1.5,1))",
"G_E___060_C18_F1_PI_SE_CS_SP_CO_S0Y": "(1.RelevanceLevelWeight(ConstPrio,2,2,0,2,100,100,100,100,1.5,1.5,1))",
"G_E___060_C18_F1_PI_SE_CS_SP_S0Y": "(1.RelevanceLevelWeight(ConstPrio,2,2,0,2,100,100,100,100,1.5,1.5,1))",
"G_E___070_K02_F2_AE_S1": "(3.Refinedweight(PreferGoals,1,1,1,1.5,2),3.Refinedweight(PreferNonGoals,1,1,1,1.5,0.5),1.Clauseweight(ConstPrio,1,1,1.5),1.FIFOWeight(ConstPrio))",
"G_E___070_K07_F2_AE_Z1": "(3.Refinedweight(PreferGoals,1,1,1,1.5,2),3.Refinedweight(PreferNonGoals,1,1,1,1.5,0.5),1.Clauseweight(ConstPrio,1,1,1.5),1.FIFOWeight(ConstPrio))",
"G_E___070_K18_F1_AE_S1": "(3.Refinedweight(PreferGoals,1,1,1,1.5,2),3.Refinedweight(PreferNonGoals,1,1,1,1.5,0.5),1.Clauseweight(ConstPrio,1,1,1.5),1.FIFOWeight(ConstPrio))",
"G_E___070_K18_F1_AE_S4_CS_Y": "(3.Refinedweight(PreferGoals,1,1,1,1.5,2),3.Refinedweight(PreferNonGoals,1,1,1,1.5,0.5),1.Clauseweight(ConstPrio,1,1,1.5),1.FIFOWeight(ConstPrio))",
"G_E___070_K18_F1_AE_Z1": "(3.Refinedweight(PreferGoals,1,1,1,1.5,2),3.Refinedweight(PreferNonGoals,1,1,1,1.5,0.5),1.Clauseweight(ConstPrio,1,1,1.5),1.FIFOWeight(ConstPrio))",
"G_E___070_K18_F2_AE_S1": "(3.Refinedweight(PreferGoals,1,1,1,1.5,2),3.Refinedweight(PreferNonGoals,1,1,1,1.5,0.5),1.Clauseweight(ConstPrio,1,1,1.5),1.FIFOWeight(ConstPrio))",
"G_E___081_K07_F1_PI_AE_Z1": "(12.SymbolTypeweight(ConstPrio,7,20,0,0,1,5,4),1.FIFOWeight(ConstPrio))",
"G_E___081_K07_F2_PI_AE_51": "(12.SymbolTypeweight(ConstPrio,7,20,0,0,1,5,4),1.FIFOWeight(ConstPrio))",
"G_E___081_K07_F2_PI_AE_Z1": "(12.SymbolTypeweight(ConstPrio,7,20,0,0,1,5,4),1.FIFOWeight(ConstPrio))",
"G_E___081_K18_F1_PI_AE_S4_CS_51": "(12.SymbolTypeweight(ConstPrio,7,20,0,0,1,5,4),1.FIFOWeight(ConstPrio))",
"G_E___081_K18_F1_PI_AE_Z1": "(12.SymbolTypeweight(ConstPrio,7,20,0,0,1,5,4),1.FIFOWeight(ConstPrio))",
"G_E___082_K02_F1_PI_AE_Y": "(20.SymbolTypeweight(ConstPrio,3,18,0,5,1,5,3),1.FIFOWeight(ConstPrio))",
"G_E___082_K02_F2_PI_AE_Y": "(20.SymbolTypeweight(ConstPrio,3,18,0,5,1,5,3),1.FIFOWeight(ConstPrio))",
"G_E___084_K16_F1_PI_AE_O1": "(12.SymbolTypeweight(ConstPrio,7,20,0,0,1.5,3,4),1.FIFOWeight(ConstPrio))",
"G_E___084_K16_F1_PI_AE_Y1": "(12.SymbolTypeweight(ConstPrio,7,20,0,0,1.5,3,4),1.FIFOWeight(ConstPrio))",
"G_E___092_C01_F1_AE_CS_SP_PS_CO_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,1,2,0,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___100_C18_F1_PI_AE_Q4_CS_SP_PS_S0Y": "(10.RelevanceLevelWeight2(SimulateSOS,1,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___100_C18_F1_PI_AE_Q4_CS_SP_S0Y": "(10.RelevanceLevelWeight2(SimulateSOS,1,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___103_C18_F1_PI_AE_Q4_CS_SP_S0Y": "(5.RelevanceLevelWeight2(ConstPrio,1,2,2,2,100,100,100,300,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed))",
"G_E___107_B00_00_F1_PI_AE_Q4_CS_SP_PS_S002A": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B00_00_F1_PI_AE_Q4_CS_SP_PS_S00EN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B00_00_F1_PI_AE_Q4_CS_SP_PS_S022A": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B00_00_F1_PI_AE_Q4_CS_SP_PS_S025A": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B00_00_F1_PI_AE_Q4_CS_SP_PS_S033N": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B00_00_F1_PI_AE_Q4_CS_SP_PS_S04AN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B00_00_F1_PI_AE_Q4_CS_SP_PS_S04BN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B00_00_F1_PI_AE_Q4_CS_SP_PS_S04CN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B00_00_F1_PI_AE_Q4_CS_SP_PS_S070I": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B00_00_F1_PI_AE_Q4_CS_SP_PS_S070N": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B00_00_F1_PI_AE_Q4_CS_SP_PS_S071I": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B00_00_F1_PI_AE_Q4_CS_SP_PS_S08BN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B01_00_F1_PI_AE_Q4_CS_SP_PS_S002A": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B01_00_F1_PI_AE_Q4_CS_SP_PS_S025A": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B01_00_F1_PI_AE_Q4_CS_SP_PS_S064A": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B01_00_F1_PI_AE_Q4_CS_SP_PS_S065A": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B01_00_F1_PI_AE_Q4_CS_SP_PS_S06DN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B03_F1_PI_AE_Q4_CS_SP_PS_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B03_F1_PI_AE_Q4_CS_SP_PS_S4S": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B42_F1_PI_AE_Q4_CS_SP_PS_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B42_F1_PI_SE_Q4_CS_SP_PS_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B42_F1_PI_SE_Q4_CS_SP_PS_S0YA": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B42_F1_PI_SE_Q4_CS_SP_PS_S0YI": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_B42_F1_PI_SE_Q4_CS_SP_PS_S5PRR_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C00_02_nc_F1_PI_AE_Q4_CS_SP_PS_S002A": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C00_02_nc_F1_PI_AE_Q4_CS_SP_PS_S00EN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C00_02_nc_F1_PI_AE_Q4_CS_SP_PS_S033N": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C00_02_nc_F1_PI_AE_Q4_CS_SP_PS_S064A": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C00_02_nc_F1_PI_AE_Q4_CS_SP_PS_S070I": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C00_02_nc_F1_PI_AE_Q4_CS_SP_PS_S071I": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C00_02_nc_F1_PI_AE_Q4_CS_SP_PS_S083N": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C00_02_nc_F1_PI_AE_Q4_CS_SP_PS_S08BN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C09_12_F1_PI_AE_Q4_CS_SP_PS_S00EN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C09_12_F1_PI_AE_Q4_CS_SP_PS_S04AN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C09_12_F1_PI_AE_Q4_CS_SP_PS_S04BN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C09_12_F1_PI_AE_Q4_CS_SP_PS_S063N": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C09_12_F1_PI_AE_Q4_CS_SP_PS_S070I": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C09_12_F1_PI_AE_Q4_CS_SP_PS_S083N": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C12_02_nc_F1_PI_AE_Q4_CS_SP_PS_S025A": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C12_02_nc_F1_PI_AE_Q4_CS_SP_PS_S033N": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C12_02_nc_F1_PI_AE_Q4_CS_SP_PS_S04CN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C12_02_nc_F1_PI_AE_Q4_CS_SP_PS_S064A": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C12_02_nc_F1_PI_AE_Q4_CS_SP_PS_S065A": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C12_02_nc_F1_PI_AE_Q4_CS_SP_PS_S070N": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C12_11_nc_F1_PI_AE_Q4_CS_SP_PS_S000A": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C12_11_nc_F1_PI_AE_Q4_CS_SP_PS_S00EN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C12_11_nc_F1_PI_AE_Q4_CS_SP_PS_S033N": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C12_11_nc_F1_PI_AE_Q4_CS_SP_PS_S04AN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C12_11_nc_F1_PI_AE_Q4_CS_SP_PS_S04BN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C12_11_nc_F1_PI_AE_Q4_CS_SP_PS_S04CN": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C12_11_nc_F1_PI_AE_Q4_CS_SP_PS_S065A": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C12_11_nc_F1_PI_AE_Q4_CS_SP_PS_S070I": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C12_11_nc_F1_PI_AE_Q4_CS_SP_PS_S071I": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C18_F1_AE_Q4_CS_SP_CO_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C18_F1_AE_Q4_CS_SP_PS_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C18_F1_AE_Q4_CS_SP_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C36_F1_PI_AE_Q4_CS_SP_PS_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C37_F1_PI_AE_Q4_CS_SP_PS_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C41_F1_PI_AE_CS_SP_PS_S4S": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C41_F1_PI_AE_Q4_CS_SP_PS_CO_S4S": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C41_F1_PI_AE_Q4_CS_SP_PS_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C41_F1_PI_AE_Q4_CS_SP_PS_S2U": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C41_F1_PI_AE_Q4_CS_SP_PS_S4S": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C45_F1_AE_Q4_CS_SP_PS_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C45_F1_PI_AE_Q7_CS_SP_PS_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___107_C48_F1_PI_AE_Q4_CS_SP_PS_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___110_C45_F1_PI_AE_Q4_CS_SP_PS_S4S": "(4.RelevanceLevelWeight2(PreferGoals,1,2,1,2,100,100,100,400,1.5,1.5,1),3.RelevanceLevelWeight2(PreferNonGoals,0,2,1,2,100,100,100,400,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___110_C45_F1_PI_SE_CS_SP_PS_S4S": "(4.RelevanceLevelWeight2(PreferGoals,1,2,1,2,100,100,100,400,1.5,1.5,1),3.RelevanceLevelWeight2(PreferNonGoals,0,2,1,2,100,100,100,400,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___110_C45_F1_PI_SE_Q4_CS_SP_PS_S4S": "(4.RelevanceLevelWeight2(PreferGoals,1,2,1,2,100,100,100,400,1.5,1.5,1),3.RelevanceLevelWeight2(PreferNonGoals,0,2,1,2,100,100,100,400,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"G_E___200_B02_F1_AE_CS_SP_PI_S0S": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_B02_F1_AE_CS_SP_PI_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_B02_F1_AE_CS_SP_PI_S2k": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_B02_F1_SE_CS_SP_PI_S0S": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_B02_F1_SE_CS_SP_PI_S2k": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_B07_F1_AE_CS_SP_PI_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_C18_F1_AE_CS_SP_PI_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_C41_F1_AE_CS_SP_PI_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_C45_F1_AE_CS_SP_PI_CO_S0V": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_C45_F1_AE_CS_SP_PI_PS_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_C45_F1_AE_CS_SP_PI_S0S": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_C45_F1_AE_CS_SP_PI_S0V": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_C45_F1_AE_CS_SP_PI_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_C45_F1_AE_CS_SP_PI_S2S": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_C45_F1_AE_CS_SP_PI_S4S": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_C45_F1_SE_CS_SP_PI_CO_S0V": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___200_C45_F1_SE_CS_SP_PI_CO_S5PRR_S0V": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___201_C45_F1_AE_CS_SP_PI_CO_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),6.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___201_C45_F1_AE_CS_SP_PI_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),6.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___204_C45_F1_AE_CS_SP_PI_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___205_C45_F1_AE_CS_SP_PI_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),6.ConjectureRelativeSymbolWeight(SimulateSOS,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),8.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_B07_F1_AE_CS_SP_PI_PS_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_B07_F1_AE_CS_SP_PS_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C01_F1_AE_CS_SP_PI_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C01_F1_SE_CS_SP_PI_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C01_F1_SE_CS_SP_PI_S5PRR_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_AE_CS_SP_PI_CO_S00": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_AE_CS_SP_PI_CO_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_AE_CS_SP_PI_PS_S0S": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_AE_CS_SP_PI_PS_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_AE_CS_SP_PI_PS_S0i": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_AE_CS_SP_PI_PS_S2U": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_AE_CS_SP_PI_PS_S3S": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_AE_CS_SP_PI_S0a": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_AE_CS_SP_PI_S0e": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_AE_CS_SP_PS_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_AE_CS_SP_PS_S3S": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_AE_Q7_CS_SP_CO_PS_S00": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_SE_CS_SP_PI_PS_S00A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_SE_CS_SP_PI_PS_S00I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_SE_CS_SP_PI_PS_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_SE_CS_SP_PI_PS_S2S": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_SE_CS_SP_PI_PS_S2SA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_SE_CS_SP_PI_PS_S2SI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_SE_CS_SP_PI_PS_S5PRR_RG_S04AN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_SE_CS_SP_PI_PS_S5PRR_RG_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_SE_CS_SP_PI_PS_S5PRR_S2S": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_SE_CS_SP_PI_PS_S5PRR_S2SI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_SP_PI_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C18_F1_SP_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C41_F1_AE_CS_SP_PI_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___207_C41_F1_AE_CS_SP_PI_S4S": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___208_B00_00_F1_AE_ND_CS_SP_PS_S00": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B00_00_F1_SE_CS_SP_PS_S00EN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B00_00_F1_SE_CS_SP_PS_S022A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B00_00_F1_SE_CS_SP_PS_S033N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B00_00_F1_SE_CS_SP_PS_S04BN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B00_00_F1_SE_CS_SP_PS_S063N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B00_00_F1_SE_CS_SP_PS_S064A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B00_00_F1_SE_CS_SP_PS_S070I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B00_00_F1_SE_CS_SP_PS_S071I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B00_00_F1_SE_CS_SP_PS_S083N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B00_00_F1_SE_CS_SP_PS_S5PRR_S00EN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B01_00_F1_AE_ND_CS_SP_PS_S00": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B01_00_F1_SE_CS_SP_PS_S024A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B01_00_F1_SE_CS_SP_PS_S033N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B01_00_F1_SE_CS_SP_PS_S04AN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B01_00_F1_SE_CS_SP_PS_S04CN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B01_00_F1_SE_CS_SP_PS_S064A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B01_00_F1_SE_CS_SP_PS_S065A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B01_00_F1_SE_CS_SP_PS_S070I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B01_00_F1_SE_CS_SP_PS_S083N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B02_F1_AE_CS_SP_PS_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B07_F1_AE_CS_SP_PS_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B07_F1_AE_CS_SP_PS_S5PRR_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B07_F1_S5PRR_SE_CS_SP_PS_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B07_F1_SE_CS_SP_PS_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B07_F1_SE_CS_SP_PS_S2q": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B07_F1_SE_CS_SP_PS_S4d": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_B08_00_F1_AE_CS_SP_S00": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C00_02_nc_F1_SE_CS_SP_PS_S023A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C00_02_nc_F1_SE_CS_SP_PS_S033N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C00_02_nc_F1_SE_CS_SP_PS_S04CN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C00_02_nc_F1_SE_CS_SP_PS_S063N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C00_02_nc_F1_SE_CS_SP_PS_S064A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C00_02_nc_F1_SE_CS_SP_PS_S071I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C09_12_F1_SE_CS_SP_PS_S000A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C09_12_F1_SE_CS_SP_PS_S002A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C09_12_F1_SE_CS_SP_PS_S033N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C09_12_F1_SE_CS_SP_PS_S064A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C09_12_F1_SE_CS_SP_PS_S070I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_02_nc_F1_SE_CS_SP_PS_S002A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_02_nc_F1_SE_CS_SP_PS_S00EN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_02_nc_F1_SE_CS_SP_PS_S022A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_02_nc_F1_SE_CS_SP_PS_S064A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_02_nc_F1_SE_CS_SP_PS_S06DN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_02_nc_F1_SE_CS_SP_PS_S071I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_02_nc_F1_SE_CS_SP_PS_S5PRR_RG_S070I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_11_nc_F1_SE_CS_SP_PS_S002A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_11_nc_F1_SE_CS_SP_PS_S00EN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_11_nc_F1_SE_CS_SP_PS_S024A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_11_nc_F1_SE_CS_SP_PS_S033N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_11_nc_F1_SE_CS_SP_PS_S04BN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_11_nc_F1_SE_CS_SP_PS_S04CN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_11_nc_F1_SE_CS_SP_PS_S08BN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_11_nc_F1_SE_CS_SP_PS_S5PRR_RG_S04AN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C12_11_nc_F1_SE_CS_SP_PS_S5PRR_S04BN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_AE_CS_SP_PI_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_AE_CS_SP_PI_S0a": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_AE_CS_SP_PS_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_AE_CS_SP_PS_S0a": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_AE_CS_SP_PS_S3S": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S00": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S000A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S000N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S002A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S002I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S003A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S003I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S003N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S005A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S005I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S006A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S006I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S006N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S007A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S007I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S007N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S008A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S008N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S00AA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S00AI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S00BA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S00BN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S00CI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S00CN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S00DA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S00DI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S00EI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S00EN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S00FI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S00FN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S010A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S010N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S011A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S011I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S011N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S013A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S013I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S013N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S014A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S014I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S014N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S015I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S016I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S017A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S018I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S018N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S019A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S01AA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S01AI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S01AN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S01BA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S01BI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S01BN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S01CA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S01CI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S01CN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S01DA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S01DI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S01EI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S01FN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S020N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S021I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S021N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S022A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S022I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S022N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S023A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S023I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S024I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S025I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S026I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S027I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S027N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S028I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S029I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S029N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S02AI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S02BA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S02BN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S02CA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S02CN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S02DI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S02DN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S02FA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S02FI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S02FN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S030A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S031A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S031N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S032I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S032N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S033A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S033I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S033N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S034N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S035A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S035I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S035N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S036I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S036N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S037I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S037N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S038I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S038N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S039A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S039I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S039N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S03AA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S03AN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S03BA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S03BI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S03CI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S03CN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S03DI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S03DN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S03EA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S03EI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S03FI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S041A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S041N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S042A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S043A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S043I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S043N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S044A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S044I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S044N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S045I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S047A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S048A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S048I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S04AA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S04AI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S04AN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S04BI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S04BN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S04CA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S04DN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S04EA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S04EI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S04EN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S04FA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S04FI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S050A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S050I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S050N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S051A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S051I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S051N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S053A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S053I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S054N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S056A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S056N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S057A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S057I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S058I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S058N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S059A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S059I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S05AA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S05AI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S05AN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S05BI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S05BN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S05CN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S05DA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S05DI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S05DN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S05EA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S05EI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S05FA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S05FI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S05FN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S060A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S060I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S060N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S061N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S062A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S063N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S064I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S064N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S065A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S065I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S065N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S066A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S067N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S068N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S069I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S069N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S06CA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S06EI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S06FI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S070I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S072A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S072N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S073A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S073N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S074A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S075A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S075I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S075N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S076A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S076I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S077A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S077I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S078I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S078N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S079I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S079N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S07BA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S07BN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S07CA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S07DN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S07EA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S07EI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S07FI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S07FN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S080I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S080N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S081A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S081I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S081N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S082A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S083A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S083I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S083N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S084N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S085A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S085I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S085N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S086A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S086I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S086N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S087N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S088A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S088I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S088N": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S089A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S089I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S08AA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S08BA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S08BI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S08BN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S08CI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S08CN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S0U": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S0YP": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S0Y_ni": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S2g": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S2i": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S2j": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S2k": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S2m": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S2n": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S2o": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S2r": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S2s": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S3b": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S4Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S4b": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S4c": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S4d": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S5PRR_RG_S04AI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S5PRR_RG_S04AN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S5PRR_RG_S04BN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S5PRR_RG_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S5PRR_S002A": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S5PRR_S059I": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S5PRR_S2m": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S5PRR_S2mA": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S5PRR_S2mI": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S5PRR_S2v": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_S5PRR_S4d": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C18_F1_SE_CS_SP_PS_TT_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C41_F1_AE_CS_SP_PS_S00": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___208_C47_F1_SE_CS_SP_PS_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"G_E___210_C18_F1_AE_CS_SP_PI_S0Y": "(1.Clauseweight(ConstPrio,2,1,1))",
"G_E___211_C18_F1_AE_CS_SP_S0Y": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),2.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,1,1,2,1.5,2))",
"G_E___213_C18_F1_AE_CS_SP_PI_S0Y": "(1.Clauseweight(ConstPrio,1,1,1))",
"G_E___215_C46_F1_AE_CS_SP_PS_S0Y": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___215_C46_F1_AE_CS_SP_PS_S2S": "(10.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(ConstPrio))",
"G_E___300_C01_F1_SE_CS_SP_S0Y": "(1.ConjectureGeneralSymbolWeight(SimulateSOS,488,104,105,32,173,0,327,3.63456933832,1.43436931217,1),1.FIFOWeight(PreferProcessed),8.Clauseweight(PreferUnitGroundGoals,1,1,0.492432663985),3.Refinedweight(PreferGoals,2,4,7,4.94895515161,6.55154057122),2.ConjectureRelativeSymbolWeight(ConstPrio,0.0640406356983,67,160,111,25,3.13817216288,2.75641029012,1))",
"G_E___300_C01_S00": "(1.ConjectureGeneralSymbolWeight(SimulateSOS,488,104,105,32,173,0,327,3.63456933832,1.43436931217,1),1.FIFOWeight(PreferProcessed),8.Clauseweight(PreferUnitGroundGoals,1,1,0.492432663985),3.Refinedweight(PreferGoals,2,4,7,4.94895515161,6.55154057122),2.ConjectureRelativeSymbolWeight(ConstPrio,0.0640406356983,67,160,111,25,3.13817216288,2.75641029012,1))",
"G_E___300_C01_S5PRR_S00": "(1.ConjectureGeneralSymbolWeight(SimulateSOS,488,104,105,32,173,0,327,3.63456933832,1.43436931217,1),1.FIFOWeight(PreferProcessed),8.Clauseweight(PreferUnitGroundGoals,1,1,0.492432663985),3.Refinedweight(PreferGoals,2,4,7,4.94895515161,6.55154057122),2.ConjectureRelativeSymbolWeight(ConstPrio,0.0640406356983,67,160,111,25,3.13817216288,2.75641029012,1))",
"G_E___300_C18_F1_SE_CS_SP_PS_S0Y": "(1.ConjectureGeneralSymbolWeight(SimulateSOS,488,104,105,32,173,0,327,3.63456933832,1.43436931217,1),1.FIFOWeight(PreferProcessed),8.Clauseweight(PreferUnitGroundGoals,1,1,0.492432663985),3.Refinedweight(PreferGoals,2,4,7,4.94895515161,6.55154057122),2.ConjectureRelativeSymbolWeight(ConstPrio,0.0640406356983,67,160,111,25,3.13817216288,2.75641029012,1))",
"G_E___300_C18_F1_SE_CS_SP_S0Y": "(1.ConjectureGeneralSymbolWeight(SimulateSOS,488,104,105,32,173,0,327,3.63456933832,1.43436931217,1),1.FIFOWeight(PreferProcessed),8.Clauseweight(PreferUnitGroundGoals,1,1,0.492432663985),3.Refinedweight(PreferGoals,2,4,7,4.94895515161,6.55154057122),2.ConjectureRelativeSymbolWeight(ConstPrio,0.0640406356983,67,160,111,25,3.13817216288,2.75641029012,1))",
"G_E___300_C18_F1_URBAN_S0Y": "(1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.1,5,0,0.1,1,4),1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.5,100,0,0.2,0.2,4),1.Refinedweight(ConstPrio,4,300,4,4,0.7),1.RelevanceLevelWeight2(PreferProcessed,0,1,2,1,1,1,200,200,2.5,9999.9,9999.9),1.StaggeredWeight(DeferSOS,1),1.SymbolTypeweight(DeferSOS,18,7,-2,5,9999.9,2,1.5),2.Clauseweight(ConstPrio,20,9999,4),2.ConjectureSymbolWeight(DeferSOS,9999,20,50,-1,50,3,3,0.5),2.StaggeredWeight(DeferSOS,2))",
"G_E___301_C18_F1_URBAN_S0Y": "(1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.1,5,0,0.1,1,4),1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.5,100,0,0.2,0.2,4),1.Refinedweight(ConstPrio,4,300,4,4,0.7),1.RelevanceLevelWeight2(PreferProcessed,0,1,2,1,1,1,200,200,2.5,9999.9,9999.9),1.StaggeredWeight(SimulateSOS,1),1.SymbolTypeweight(DeferSOS,18,7,-2,5,9999.9,2,1.5),2.Clauseweight(ConstPrio,20,9999,4),2.ConjectureSymbolWeight(SimulateSOS,9999,20,50,-1,50,3,3,0.5),2.StaggeredWeight(DeferSOS,2))",
"G_E___301_C18_F1_URBAN_S5PRR_RG_S04AN": "(1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.1,5,0,0.1,1,4),1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.5,100,0,0.2,0.2,4),1.Refinedweight(ConstPrio,4,300,4,4,0.7),1.RelevanceLevelWeight2(PreferProcessed,0,1,2,1,1,1,200,200,2.5,9999.9,9999.9),1.StaggeredWeight(SimulateSOS,1),1.SymbolTypeweight(DeferSOS,18,7,-2,5,9999.9,2,1.5),2.Clauseweight(ConstPrio,20,9999,4),2.ConjectureSymbolWeight(SimulateSOS,9999,20,50,-1,50,3,3,0.5),2.StaggeredWeight(DeferSOS,2))",
"G_E___301_C18_F1_URBAN_S5PRR_RG_S04BN": "(1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.1,5,0,0.1,1,4),1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.5,100,0,0.2,0.2,4),1.Refinedweight(ConstPrio,4,300,4,4,0.7),1.RelevanceLevelWeight2(PreferProcessed,0,1,2,1,1,1,200,200,2.5,9999.9,9999.9),1.StaggeredWeight(SimulateSOS,1),1.SymbolTypeweight(DeferSOS,18,7,-2,5,9999.9,2,1.5),2.Clauseweight(ConstPrio,20,9999,4),2.ConjectureSymbolWeight(SimulateSOS,9999,20,50,-1,50,3,3,0.5),2.StaggeredWeight(DeferSOS,2))",
"G_E___301_C18_F1_URBAN_S5PRR_RG_S070I": "(1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.1,5,0,0.1,1,4),1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.5,100,0,0.2,0.2,4),1.Refinedweight(ConstPrio,4,300,4,4,0.7),1.RelevanceLevelWeight2(PreferProcessed,0,1,2,1,1,1,200,200,2.5,9999.9,9999.9),1.StaggeredWeight(SimulateSOS,1),1.SymbolTypeweight(DeferSOS,18,7,-2,5,9999.9,2,1.5),2.Clauseweight(ConstPrio,20,9999,4),2.ConjectureSymbolWeight(SimulateSOS,9999,20,50,-1,50,3,3,0.5),2.StaggeredWeight(DeferSOS,2))",
"G_E___301_C18_F1_URBAN_S5PRR_RG_S0Y": "(1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.1,5,0,0.1,1,4),1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.5,100,0,0.2,0.2,4),1.Refinedweight(ConstPrio,4,300,4,4,0.7),1.RelevanceLevelWeight2(PreferProcessed,0,1,2,1,1,1,200,200,2.5,9999.9,9999.9),1.StaggeredWeight(SimulateSOS,1),1.SymbolTypeweight(DeferSOS,18,7,-2,5,9999.9,2,1.5),2.Clauseweight(ConstPrio,20,9999,4),2.ConjectureSymbolWeight(SimulateSOS,9999,20,50,-1,50,3,3,0.5),2.StaggeredWeight(DeferSOS,2))",
"G_E___301_C18_F1_URBAN_S5PRR_S0Y": "(1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.1,5,0,0.1,1,4),1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.5,100,0,0.2,0.2,4),1.Refinedweight(ConstPrio,4,300,4,4,0.7),1.RelevanceLevelWeight2(PreferProcessed,0,1,2,1,1,1,200,200,2.5,9999.9,9999.9),1.StaggeredWeight(SimulateSOS,1),1.SymbolTypeweight(DeferSOS,18,7,-2,5,9999.9,2,1.5),2.Clauseweight(ConstPrio,20,9999,4),2.ConjectureSymbolWeight(SimulateSOS,9999,20,50,-1,50,3,3,0.5),2.StaggeredWeight(DeferSOS,2))",
"G_E___302_C18_F1_URBAN_RG_S04BN": "(1.ConjectureTermPrefixWeight(ConstPrio,1,3,0.1,5,0,0.1,1,4),1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.5,100,0,0.2,0.2,4),1.Refinedweight(ConstPrio,4,300,4,4,0.7),1.RelevanceLevelWeight2(PreferProcessed,0,1,2,1,1,1,200,200,2.5,9999.9,9999.9),1.StaggeredWeight(SimulateSOS,1),1.SymbolTypeweight(SimulateSOS,18,7,-2,5,9999.9,2,1.5),2.Clauseweight(ConstPrio,20,9999,4),2.ConjectureSymbolWeight(SimulateSOS,9999,20,50,-1,50,3,3,0.5),2.StaggeredWeight(DeferSOS,2))",
"G_E___302_C18_F1_URBAN_S0Y": "(1.ConjectureTermPrefixWeight(ConstPrio,1,3,0.1,5,0,0.1,1,4),1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.5,100,0,0.2,0.2,4),1.Refinedweight(ConstPrio,4,300,4,4,0.7),1.RelevanceLevelWeight2(PreferProcessed,0,1,2,1,1,1,200,200,2.5,9999.9,9999.9),1.StaggeredWeight(SimulateSOS,1),1.SymbolTypeweight(SimulateSOS,18,7,-2,5,9999.9,2,1.5),2.Clauseweight(ConstPrio,20,9999,4),2.ConjectureSymbolWeight(SimulateSOS,9999,20,50,-1,50,3,3,0.5),2.StaggeredWeight(DeferSOS,2))",
"G_E___302_C18_F1_URBAN_S5PRR_RG_S04BN": "(1.ConjectureTermPrefixWeight(ConstPrio,1,3,0.1,5,0,0.1,1,4),1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.5,100,0,0.2,0.2,4),1.Refinedweight(ConstPrio,4,300,4,4,0.7),1.RelevanceLevelWeight2(PreferProcessed,0,1,2,1,1,1,200,200,2.5,9999.9,9999.9),1.StaggeredWeight(SimulateSOS,1),1.SymbolTypeweight(SimulateSOS,18,7,-2,5,9999.9,2,1.5),2.Clauseweight(ConstPrio,20,9999,4),2.ConjectureSymbolWeight(SimulateSOS,9999,20,50,-1,50,3,3,0.5),2.StaggeredWeight(DeferSOS,2))",
"G_E___302_C18_F1_URBAN_S5PRR_RG_S0Y": "(1.ConjectureTermPrefixWeight(ConstPrio,1,3,0.1,5,0,0.1,1,4),1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.5,100,0,0.2,0.2,4),1.Refinedweight(ConstPrio,4,300,4,4,0.7),1.RelevanceLevelWeight2(PreferProcessed,0,1,2,1,1,1,200,200,2.5,9999.9,9999.9),1.StaggeredWeight(SimulateSOS,1),1.SymbolTypeweight(SimulateSOS,18,7,-2,5,9999.9,2,1.5),2.Clauseweight(ConstPrio,20,9999,4),2.ConjectureSymbolWeight(SimulateSOS,9999,20,50,-1,50,3,3,0.5),2.StaggeredWeight(DeferSOS,2))",
"G_E___302_C18_F1_URBAN_S5PRR_S0Y": "(1.ConjectureTermPrefixWeight(ConstPrio,1,3,0.1,5,0,0.1,1,4),1.ConjectureTermPrefixWeight(DeferSOS,1,3,0.5,100,0,0.2,0.2,4),1.Refinedweight(ConstPrio,4,300,4,4,0.7),1.RelevanceLevelWeight2(PreferProcessed,0,1,2,1,1,1,200,200,2.5,9999.9,9999.9),1.StaggeredWeight(SimulateSOS,1),1.SymbolTypeweight(SimulateSOS,18,7,-2,5,9999.9,2,1.5),2.Clauseweight(ConstPrio,20,9999,4),2.ConjectureSymbolWeight(SimulateSOS,9999,20,50,-1,50,3,3,0.5),2.StaggeredWeight(DeferSOS,2))",
"G_E___303_C18_F1_URBAN_S0Y": "(1.ConjectureTermPrefixWeight(ConstPrio,1,3,0.1,5,0,0.1,1,4),1.ConjectureTermPrefixWeight(ConstPrio,1,3,0.5,100,0,0.2,0.2,4),1.Refinedweight(ConstPrio,4,300,4,4,0.7),1.RelevanceLevelWeight2(ConstPrio,0,1,2,1,1,1,200,200,2.5,9999.9,9999.9),1.StaggeredWeight(ConstPrio,1),1.SymbolTypeweight(ConstPrio,18,7,-2,5,9999.9,2,1.5),2.Clauseweight(ConstPrio,20,9999,4),2.ConjectureSymbolWeight(ConstPrio,9999,20,50,-1,50,3,3,0.5),2.StaggeredWeight(ConstPrio,2))",
"G_N___023_B07_F1_SP_PI_Q7_CS_SE_S0Y": "(12.Clauseweight(ConstPrio,3,1,1),1.FIFOWeight(ConstPrio))",
"G_N___023_B07_F1_SP_PI_Q7_CS_SP_CO_S0Y": "(12.Clauseweight(ConstPrio,3,1,1),1.FIFOWeight(ConstPrio))",
"G_N___023_B07_F1_SP_PI_Q7_CS_SP_CO_S5PRR_S0Y1": "(12.Clauseweight(ConstPrio,3,1,1),1.FIFOWeight(ConstPrio))",
"G_N___023_K07_F1_PI_S4_CS_Y": "(12.Clauseweight(ConstPrio,3,1,1),1.FIFOWeight(ConstPrio))",
"G_N___023_K07_F2_PI_S4_O": "(12.Clauseweight(ConstPrio,3,1,1),1.FIFOWeight(ConstPrio))",
"G_N___023_K07_F2_PI_S4_O1": "(12.Clauseweight(ConstPrio,3,1,1),1.FIFOWeight(ConstPrio))",
"G_N___023_K07_F2_PI_S4_P": "(12.Clauseweight(ConstPrio,3,1,1),1.FIFOWeight(ConstPrio))",
"G_N___023_K07_F2_PI_S4_P3": "(12.Clauseweight(ConstPrio,3,1,1),1.FIFOWeight(ConstPrio))",
"G_N___023_K07_F2_PI_S7_P3": "(12.Clauseweight(ConstPrio,3,1,1),1.FIFOWeight(ConstPrio))",
"G_N___023_K18_F1_PI_S4_O": "(12.Clauseweight(ConstPrio,3,1,1),1.FIFOWeight(ConstPrio))",
"G_N___023_K18_F2_PI_S4_O1": "(12.Clauseweight(ConstPrio,3,1,1),1.FIFOWeight(ConstPrio))",
"G_N___070_K01_F2_PI_S4_AE_V": "(3.Refinedweight(PreferGoals,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,0.5),1.Clauseweight(ConstPrio,1,1,1.5),1.FIFOWeight(ConstPrio))",
"G_N___070_K16_F1_PI_S4_AE_S1": "(3.Refinedweight(PreferGoals,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,0.5),1.Clauseweight(ConstPrio,1,1,1.5),1.FIFOWeight(ConstPrio))",
"G_N___070_K16_F1_PI_S4_AE_V": "(3.Refinedweight(PreferGoals,1,1,2,1.5,2),3.Refinedweight(PreferNonGoals,1,1,2,1.5,0.5),1.Clauseweight(ConstPrio,1,1,1.5),1.FIFOWeight(ConstPrio))",
"G_____0001_C08_07_nc_F1_AE_CS_SP_ND_PS_S00": "(1.FIFOWeight(ConstPrio))",
"G_____0003_C18_F1_SE_CS_SP_S0Y": "(1.Clauseweight(ConstPrio,2,1,1))",
"G_____0008_C18_F1_SE_CS_SP_S0Y": "(1.Refinedweight(ConstPrio,2,1,2,2,1))",
"G_____0010_evo": "(1.ConjectureGeneralSymbolWeight(SimulateSOS,488,104,105,32,173,0,327,3.63456933832,1.43436931217,1),1.FIFOWeight(PreferProcessed),8.Clauseweight(PreferUnitGroundGoals,1,1,0.492432663985),3.Refinedweight(PreferGoals,2,4,7,4.94895515161,6.55154057122),2.ConjectureRelativeSymbolWeight(ConstPrio,0.0640406356983,67,160,111,25,3.13817216288,2.75641029012,1))",
"G_____0012_C18_F1_SE_CS_SP_S5PRR_RG_S04AN": "(1.Refinedweight(ConstPrio,2,1,1.5,1,1))",
"G_____0012_C18_F1_SE_CS_SP_S5PRR_RG_S0Y": "(1.Refinedweight(ConstPrio,2,1,1.5,1,1))",
"G_____0014_C01_20_nc_F1_SE_CS_SP_S0Y": "(10.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"G_____0014_C10_23_nc_F1_SE_CS_SP_S0Y": "(10.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"G_____0014_C18_F1_SE_CS_SP_S0Y": "(10.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"G_____0015_C18_F1_SE_CS_SP_S0Y": "(1.ConjectureGeneralSymbolWeight(ConstPrio,15,3,3,1,5,0,10,3.6,1.4,1))",
"G_____0016_C18_F1_SE_CS_SP_S0Y": "(5.ConjectureGeneralSymbolWeight(ConstPrio,15,3,3,1,5,0,10,3.6,1.4,1),1.FIFOWeight(ConstPrio))",
"G_____0017_C18_F1_PI_SE_CS_SP_S4Y": "(15.Clauseweight(ConstPrio,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____0017_C18_F1_SE_CS_SP_S4Y": "(15.Clauseweight(ConstPrio,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____0019_C18_F1_PI_SE_CS_SP_S0Y": "(1.ConjectureGeneralSymbolWeight(ConstPrio,1,1,1,3,3,3,0,1,1,1))",
"G_____0021_C18_F1_SE_CS_SP_S0Y": "(1.ConjectureRelativeSymbolWeight(ConstPrio,0.07,3,6,4,1,1,1,1))",
"G_____0021_C18_F1_SE_CS_SP_S4S": "(1.ConjectureRelativeSymbolWeight(ConstPrio,0.07,3,6,4,1,1,1,1))",
"G_____0021_C18_F1_SE_CS_SP_S5PRR_S0Y": "(1.ConjectureRelativeSymbolWeight(ConstPrio,0.07,3,6,4,1,1,1,1))",
"G_____0026_C18_F1_SE_CS_SP_S0Y": "(15.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"G_____0026_C18_F1_SE_CS_SP_S4S": "(15.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"G_____0029_C18_F1_PI_SE_CS_SP_S0Y": "(1.Clauseweight(ConstPrio,1,1,2))",
"G_____0029_C18_F1_SE_CS_SP_S0Y": "(1.Clauseweight(ConstPrio,1,1,2))",
"G_____0031_C18_F1_SE_CS_SP_S0Y": "(1.ConjectureGeneralSymbolWeight(ConstPrio,3,3,3,1,1,1,4,1,1,1))",
"G_____0033_C18_F1_SE_CS_SP_S0Y": "(1.ConjectureGeneralSymbolWeight(ConstPrio,1,1,1,3,3,3,4,1,1,2))",
"G_____0034_C18_F1_SE_CS_SP_S0Y": "(1.ConjectureGeneralSymbolWeight(ConstPrio,1,1,1,5,5,5,5,1,1,1))",
"G_____1023__C12_02_nc_F1_AE_CS_SP_S2S": "(1.Diversityweight(ConstPrio,1,1,1.5,1,1,-1.000000,1.000000,0.000000,0.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____1027__C12_02_nc_F1_AE_CS_SP_S2S": "(1.Diversityweight(ConstPrio,1,1,1.5,1,1,-1.000000,1.000000,1.000000,1.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____1110__C12_02_nc_F1_AE_CS_SP_S2S": "(5.Diversityweight(ConstPrio,1,1,1.5,1,1,0.000000,-1.000000,-1.000000,0.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____1233__C12_02_nc_F1_AE_CS_SP_S2S": "(10.Diversityweight(ConstPrio,1,1,1.5,1,1,1.000000,0.000000,1.000000,0.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____2135__C12_02_nc_F1_AE_CS_SP_S2S": "(5.Diversityweight(ConstPrio,1,1,1.5,1,1,0.000000,1.000000,1.000000,1.000000),1.RandomWeight(ConstPrio,20,1,1))",
"G_____2275__C12_02_nc_F1_AE_CS_SP_S2S": "(20.Diversityweight(ConstPrio,1,1,1.5,1,1,0.000000,-1.000000,0.000000,0.000000),1.RandomWeight(ConstPrio,20,1,1))",
"G_____X1136__C12_02_nc_F1_AE_CS_SP_S2S": "(10.Diversityweight(ConstPrio,1,1,1.5,1,1,1.000000,-1.000000,0.000000,2.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____X1276__C12_02_nc_F1_AE_CS_SP_RG_S04AN": "(20.Diversityweight(ConstPrio,1,1,1.5,1,1,-1.000000,0.000000,-1.000000,2.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____X1276__C12_02_nc_F1_AE_CS_SP_S2S": "(20.Diversityweight(ConstPrio,1,1,1.5,1,1,-1.000000,0.000000,-1.000000,2.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____X1372__C12_02_nc_F1_AE_CS_SP_S2S": "(20.Diversityweight(ConstPrio,1,1,1.5,1,1,0.000000,2.000000,-1.000000,2.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____X1495__C12_02_nc_F1_AE_CS_SP_S2S": "(20.Diversityweight(ConstPrio,1,1,1.5,1,1,2.000000,1.000000,2.000000,1.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____Y1006__C12_02_nc_F1_AE_CS_SP_S2S": "(10.Diversityweight(ConstPrio,1,1,1.5,1,1,-2.000000,-2.000000,0.000000,0.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____Y1145__C12_02_nc_F1_AE_CS_SP_S2S": "(10.Diversityweight(ConstPrio,1,1,1.5,1,1,2.000000,0.000000,-2.000000,-2.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____Y1156__C12_02_nc_F1_AE_CS_SP_S2S": "(10.Diversityweight(ConstPrio,1,1,1.5,1,1,2.000000,0.000000,2.000000,5.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____Z1014__C12_02_nc_F1_AE_CS_SP_S2S": "(10.Diversityweight(SimulateSOS,1,1,1.5,1,1,-1.000000,-1.000000,2.000000,0.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____Z1328__C12_02_nc_F1_AE_CS_SP_S2S": "(20.Diversityweight(SimulateSOS,1,1,1.5,1,1,0.000000,-1.000000,0.000000,2.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"G_____Z1357__C12_02_nc_F1_AE_CS_SP_S2S": "(20.Diversityweight(SimulateSOS,1,1,1.5,1,1,0.000000,1.000000,0.000000,-1.000000),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____011_C07_F1_PI_AE_OS_R8_SP_S1U": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_C07_F1_PI_AE_OS_S1U": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_C07_F1_PI_AE_Q4_CS_SP_PS_S0V": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_C07_F1_PI_AE_Q4_SP_S0V": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_C07_F1_PI_AE_SP_S0V": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_C07_F1_PI_SE_SP_S0V": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_C18_F1_PI_AE_SP_S2S": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_C18_F1_PI_SE_SP_S2S": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_K01_F1_PI_6": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_K01_F2_PI_6": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_K07_F1_PI_AE_OS_S1U": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_K07_F1_PI_AE_S4_CS_SP_SOV": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_K07_F1_PI_AE_SP_SOV": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_K07_F1_PI_AE_V": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_K07_F2_PI_V": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_K16_F2_PI_6": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_K16_F2_PI_S1": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____011_K18_F1_PI_5": "(8.Refinedweight(PreferGoals,1,2,2,1,0.8),8.Refinedweight(PreferNonGoals,2,1,2,3,0.8),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____013_K16_F1_PI_AE_CS_o": "(12.Refinedweight(PreferGoals,1,2,2,1,0.8),12.Refinedweight(PreferNonGoals,2,1,2,3,0.8),2.Clauseweight(ConstPrio,1,1,0.7))",
"H_____013_K16_F1_PI_AE_o": "(12.Refinedweight(PreferGoals,1,2,2,1,0.8),12.Refinedweight(PreferNonGoals,2,1,2,3,0.8),2.Clauseweight(ConstPrio,1,1,0.7))",
"H_____013_K16_F2_PI_o": "(12.Refinedweight(PreferGoals,1,2,2,1,0.8),12.Refinedweight(PreferNonGoals,2,1,2,3,0.8),2.Clauseweight(ConstPrio,1,1,0.7))",
"H_____014_K18_F1_PI_AE_CS_SP_S2S": "(12.Refinedweight(PreferGoals,1,2,2,1,0.8),12.Refinedweight(PreferNonGoals,2,1,2,3,0.8),2.ClauseWeightAge(ConstPrio,1,1,0.7,3))",
"H_____024_K18_F1_PI_AE_S4_CS_SP_S2S": "(12.SymbolTypeweight(ConstPrio,7,20,0,0,1.5,5,0.8),1.FIFOWeight(ConstPrio))",
"H_____024_K18_F1_PI_OP_AE_S4_CS_S1": "(12.SymbolTypeweight(ConstPrio,7,20,0,0,1.5,5,0.8),1.FIFOWeight(ConstPrio))",
"H_____024_K18_F1_PI_OP_AE_S4_CS_SP_S2S": "(12.SymbolTypeweight(ConstPrio,7,20,0,0,1.5,5,0.8),1.FIFOWeight(ConstPrio))",
"H_____024_K18_F2_OP_S1": "(12.SymbolTypeweight(ConstPrio,7,20,0,0,1.5,5,0.8),1.FIFOWeight(ConstPrio))",
"H_____031_K02_F1_PI_6": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K04_F1_PI_AE_S4_CS_Y": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K07_F1_PI_AE_S4_CS_6": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K07_F2_61": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K07_F2_PI_AE_5": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K07_F2_PI_AE_6": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K07_F2_PI_AE_6_csr": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K07_F2_PI_AE_S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K07_F2_PI_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K07_F2_PI_AE_T": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K07_F2_PI_AE_Z1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K09_F1_PI_AE_SP_S0Y": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K09_F1_PI_AE_V": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K09_F1_PI_AE_Z1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K09_F2_PI_AE_6": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K09_F2_PI_AE_V": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K09_F2_PI_AE_Z1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K10_F2_PI_AE_Z1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K16_F0_PI_AE_S4_CS_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K16_F0_PI_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K16_F1_PI_AE_S4_CS_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K16_F1_PI_AE_S4_CS_Y": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K16_F1_PI_AE_S4_O1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K16_F1_PI_AE_S4_P1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K16_F1_PI_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K16_F1_PI_P": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K16_F1_PI_Y": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K16_F2_PI_P": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K16_F2_PI_Y": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K18_F1_61": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K18_F1_PI_AE_61": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K18_F1_PI_AE_CS_61": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K18_F1_PI_AE_CS_SP_S3T": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K18_F1_PI_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____031_K21_F1_PI_AE_S4_O1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____032_K18_F1_PI_AE_i": "(8.Refinedweight(PreferGoals,1,2,2,3,2),8.Refinedweight(PreferNonGoals,2,1,2,3,2),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____032_K18_F1_PI_i": "(8.Refinedweight(PreferGoals,1,2,2,3,2),8.Refinedweight(PreferNonGoals,2,1,2,3,2),1.Clauseweight(ConstPrio,1,1,0.7),1.FIFOWeight(ByNegLitDist))",
"H_____035_K16_F1_PI_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.Clauseweight(ByCreationDate,-2,-1,0.5))",
"H_____035_K16_F1_PI_T1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.Clauseweight(ByCreationDate,-2,-1,0.5))",
"H_____036_K18_F1_PI_AE_S4_CS_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____036_K18_F2_PI_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____036_K18_F2_PI_AE_S4_SP_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____037_K18_F2_PI_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.StaggeredWeight(ConstPrio,1))",
"H_____039_K16_F1_PI_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.StaggeredWeight(ConstPrio,2))",
"H_____039_K18_F2_PI_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.StaggeredWeight(ConstPrio,2))",
"H_____041_K18_F1_PI_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.StaggeredWeight(ConstPrio,1.5))",
"H_____041_K18_F2_PI_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),1.Clauseweight(ConstPrio,1,1,1),1.StaggeredWeight(ConstPrio,1.5))",
"H_____042_B03_F1_AE_Q4_SP_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_B03_F1_PI_AE_SP_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_B07_F1_PI_AE_SP_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_B30_F1_PI_AE_SP_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_K16_F1_PI_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_K18_F1_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_K18_F1_PI_AE_S4_6": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_K18_F1_PI_AE_S4_61": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_K18_F1_PI_AE_S4_CB_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_K18_F1_PI_AE_S4_CS_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_K18_F1_PI_AE_S4_O1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_K18_F1_PI_AE_S4_Z1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_K18_F2_PI_AE_S4_CB_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_K19_F1_PI_AE_S4_CS_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_K19_F1_PI_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_L07_F1_PI_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_L19_F1_PI_AE_S4_CS_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____042_L19_F1_PI_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____043_K18_F1_PI_AE_S4_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.StaggeredWeight(ConstPrio,2))",
"H_____047_B01_00_F1_AE_ND_SP_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_B07_00_F1_AE_ND_SP_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_B09_00_F1_AE_R8_CS_SP_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_B10_00_F1_AE_R8_CS_SP_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_B31_F1_PI_AE_R4_CS_SP_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_B31_F1_PI_AE_R4_CS_SP_S2k": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_B31_F1_PI_AE_R4_CS_SP_S4c": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_B31_F1_PI_AE_R4_CS_SP_S4d": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_BXX_XX_F1_AE_ND_SP_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_C02_F1_PI_SE_Q12_CS_SP_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_C05_04_F1_AE_R8_CS_SP_RG_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_C09_12_F1_AE_ND_CS_SP_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_C09_12_F1_AE_ND_CS_SP_S5PRR_RG_S04AN": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_C09_12_F1_AE_ND_CS_SP_S5PRR_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_C11_20_nc_F1_AE_ND_SP_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_C12_11_nc_F1_AE_R8_CS_SP_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_C18_F1_AE_R8_CS_SP_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_C18_F1_PI_AE_R8_CS_SP_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_C45_F1_AE_R8_CS_SP_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_K18_F1_PI_AE_S4_CS_S1": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_K18_F1_PI_AE_S4_CS_SP_S2S": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____047_K18_F1_PI_AE_S4_S1": "(10.PNRefinedweight(PreferGoals,1,1,1,2,2,2,0.5),10.PNRefinedweight(PreferNonGoals,2,1,1,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"H_____048_K18_F1_PI_AE_S4_S1": "(8.Refinedweight(PreferGoals,1,2,2,2,0.5),12.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.Defaultweight(ByCreationDate))",
"H_____050_K18_F1_PI_AE_S4_S1": "(11.Refinedweight(PreferGoals,1,2,2,2,0.5),9.Refinedweight(PreferNonGoals,2,1,2,2,2),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.Defaultweight(ByCreationDate))",
"H_____072_K16_F1_PI_AE_CS_S4_S0Y": "(10.Refinedweight(PreferGroundGoals,2,1,2,1.0,1),1.Clauseweight(ConstPrio,1,1,1),1.Clauseweight(ByCreationDate,2,1,0.8))",
"H_____072_K16_F1_PI_AE_CS_S4_Y": "(10.Refinedweight(PreferGroundGoals,2,1,2,1.0,1),1.Clauseweight(ConstPrio,1,1,1),1.Clauseweight(ByCreationDate,2,1,0.8))",
"H_____072_K16_F1_PI_AE_CS_Y": "(10.Refinedweight(PreferGroundGoals,2,1,2,1.0,1),1.Clauseweight(ConstPrio,1,1,1),1.Clauseweight(ByCreationDate,2,1,0.8))",
"H_____072_K16_F2_PI_Y": "(10.Refinedweight(PreferGroundGoals,2,1,2,1.0,1),1.Clauseweight(ConstPrio,1,1,1),1.Clauseweight(ByCreationDate,2,1,0.8))",
"H_____081_B02_F1_PI_AE_S4_CS_SP_S2S": "(8.Refinedweight(PreferGoals,1,2,2,2,2),8.Refinedweight(PreferNonGoals,2,1,2,2,0.5),1.Clauseweight(PreferUnitGroundGoals,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____081_B07_F1_PI_AE_S4_CS_SP_S2S": "(8.Refinedweight(PreferGoals,1,2,2,2,2),8.Refinedweight(PreferNonGoals,2,1,2,2,0.5),1.Clauseweight(PreferUnitGroundGoals,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____081_K07_F2_PI_AE_l": "(8.Refinedweight(PreferGoals,1,2,2,2,2),8.Refinedweight(PreferNonGoals,2,1,2,2,0.5),1.Clauseweight(PreferUnitGroundGoals,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____081_K10_F1_PI_AE_S4_CS_S1": "(8.Refinedweight(PreferGoals,1,2,2,2,2),8.Refinedweight(PreferNonGoals,2,1,2,2,0.5),1.Clauseweight(PreferUnitGroundGoals,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____081_K10_F1_PI_AE_l": "(8.Refinedweight(PreferGoals,1,2,2,2,2),8.Refinedweight(PreferNonGoals,2,1,2,2,0.5),1.Clauseweight(PreferUnitGroundGoals,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____081_K10_F2_PI_AE_l": "(8.Refinedweight(PreferGoals,1,2,2,2,2),8.Refinedweight(PreferNonGoals,2,1,2,2,0.5),1.Clauseweight(PreferUnitGroundGoals,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____081_K18_F1_PI_AE_S4_CS_SP_S2S": "(8.Refinedweight(PreferGoals,1,2,2,2,2),8.Refinedweight(PreferNonGoals,2,1,2,2,0.5),1.Clauseweight(PreferUnitGroundGoals,1,1,1),1.FIFOWeight(ConstPrio))",
"H_____102_B03_F1_PI_AE_S4_CS_SP_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_B08_00_F1_PI_AE_CS_SP_PS_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_B31_F1_PI_AE_S4_CS_SP_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_BXX_XX_F1_PI_AE_CS_SP_PS_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_C00_02_nc_F1_PI_AE_CS_SP_PS_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_C01_17_nc_F1_PI_AE_CS_SP_PS_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_C03_21_nc_F1_PI_AE_CS_SP_PS_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_C06_02_nc_F1_PI_AE_CS_SP_PS_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_C06_24_nc_F1_PI_AE_CS_SP_PS_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_C18_F1_PI_AE_CS_SP_PS_S0Y": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_C18_F1_PI_AE_CS_SP_PS_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_C18_F1_PI_AE_Q4_CS_SP_S1S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_K18_F1_PI_AE_AB_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_K18_F1_PI_AE_S4_CS_O1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_K18_F1_PI_AE_S4_CS_S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_K18_F1_PI_AE_S4_CS_S1": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_K18_F1_PI_AE_S4_CS_S1S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_K18_F1_PI_AE_S4_CS_SP_S2S": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____102_K18_F1_PI_AE_S4_CS_U": "(10.Refinedweight(PreferGoals,1,2,2,2,0.5),10.Refinedweight(PreferNonGoals,2,1,2,2,2),3.OrientLMaxWeight(ConstPrio,2,1,2,1,1),2.ClauseWeightAge(ConstPrio,1,1,1,3))",
"H_____107_C06_02_nc_F1_PI_AE_Q4_CS_SP_PS_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"H_____107_C10_03_nc_F1_PI_AE_Q4_CS_SP_PS_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"H_____107_C11_03_nc_F1_PI_AE_Q4_CS_SP_PS_S0Y": "(4.RelevanceLevelWeight2(SimulateSOS,0,2,1,2,100,100,100,400,1.5,1.5,1),3.ConjectureGeneralSymbolWeight(PreferNonGoals,200,100,200,50,50,1,100,1.5,1.5,1),1.Clauseweight(PreferProcessed,1,1,1),1.FIFOWeight(PreferProcessed))",
"MinMin_NN": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"MinMin_rr": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"SAT001_MinMin_rr": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"SAT001_MinMin_x000000_rr": "(1.ConjectureRelativeSymbolWeight(SimulateSOS,0.5,100,100,100,100,1.5,1.5,1),4.ConjectureRelativeSymbolWeight(ConstPrio,0.1,100,100,100,100,1.5,1.5,1.5),1.FIFOWeight(PreferProcessed),1.ConjectureRelativeSymbolWeight(PreferNonGoals,0.5,100,100,100,100,1.5,1.5,1),4.Refinedweight(SimulateSOS,3,2,2,1.5,2))",
"SAT002_MinMin_rr": "(1.Clauseweight(ConstPrio,1,1,1))",
"SAT002_MinMin_x000000_rr": "(1.Clauseweight(ConstPrio,1,1,1))",
"SAT003_MinMin_x000000_rr": "(5.Clauseweight(ConstPrio,1,1,1),1.FIFOWeight(SimulateSOS))",
"U_____002_K04_F2_PI_S1": "(20.Refinedweight(PreferGoals,4,3,2.0,1,1),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"U_____002_K18_F1_PI_O1": "(20.Refinedweight(PreferGoals,4,3,2.0,1,1),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"U_____002_L07_F0_PI_PG_AE_S1": "(20.Refinedweight(PreferGoals,4,3,2.0,1,1),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"U_____002_L07_F1_PI_PG_AE_S1": "(20.Refinedweight(PreferGoals,4,3,2.0,1,1),5.OrientLMaxWeight(ConstPrio,2,1,2,1,1),1.FIFOWeight(ConstPrio))",
"U_____009_K18_F1_PI_AE_S1": "(20.Refinedweight(PreferGoals,2,1,2.0,1,1),5.OrientLMaxWeight(ConstPrio,2,1,4,1,1),1.FIFOWeight(ConstPrio))",
"U_____009_L02_F1_PI_CS_S1": "(20.Refinedweight(PreferGoals,2,1,2.0,1,1),5.OrientLMaxWeight(ConstPrio,2,1,4,1,1),1.FIFOWeight(ConstPrio))",
"U_____009_L02_F1_PI_S1": "(20.Refinedweight(PreferGoals,2,1,2.0,1,1),5.OrientLMaxWeight(ConstPrio,2,1,4,1,1),1.FIFOWeight(ConstPrio))",
"U_____043_B07_F1_PI_AE_CS_SP_S0Y": "(4.PNRefinedweight(PreferNonGoals,4,5,5,4,2,1,1),8.PNRefinedweight(PreferGoals,5,2,2,5,2,1,0.5),1.FIFOWeight(ConstPrio))",
"U_____043_B31_F1_PI_AE_CS_SP_S2S": "(4.PNRefinedweight(PreferNonGoals,4,5,5,4,2,1,1),8.PNRefinedweight(PreferGoals,5,2,2,5,2,1,0.5),1.FIFOWeight(ConstPrio))",
"U_____043_C18_F1_PI_AE_S4_CS_SP_S0Y": "(4.PNRefinedweight(PreferNonGoals,4,5,5,4,2,1,1),8.PNRefinedweight(PreferGoals,5,2,2,5,2,1,0.5),1.FIFOWeight(ConstPrio))",
"U_____043_K09_F1_PI_AE_CS_SP_S0Y": "(4.PNRefinedweight(PreferNonGoals,4,5,5,4,2,1,1),8.PNRefinedweight(PreferGoals,5,2,2,5,2,1,0.5),1.FIFOWeight(ConstPrio))",
"U_____043_K16_F1_PI_S1": "(4.PNRefinedweight(PreferNonGoals,4,5,5,4,2,1,1),8.PNRefinedweight(PreferGoals,5,2,2,5,2,1,0.5),1.FIFOWeight(ConstPrio))",
"U_____043_K18_F1_PI_AE_CS_Y1": "(4.PNRefinedweight(PreferNonGoals,4,5,5,4,2,1,1),8.PNRefinedweight(PreferGoals,5,2,2,5,2,1,0.5),1.FIFOWeight(ConstPrio))",
"U_____043_K18_F1_PI_O1": "(4.PNRefinedweight(PreferNonGoals,4,5,5,4,2,1,1),8.PNRefinedweight(PreferGoals,5,2,2,5,2,1,0.5),1.FIFOWeight(ConstPrio))",
"U_____043_K18_F2_PI_AE_CS_SP_S0Y": "(4.PNRefinedweight(PreferNonGoals,4,5,5,4,2,1,1),8.PNRefinedweight(PreferGoals,5,2,2,5,2,1,0.5),1.FIFOWeight(ConstPrio))",
"U_____043_K19_F1_PI_AE_CS_SP_S0Y": "(4.PNRefinedweight(PreferNonGoals,4,5,5,4,2,1,1),8.PNRefinedweight(PreferGoals,5,2,2,5,2,1,0.5),1.FIFOWeight(ConstPrio))",

}


precs = [
    "unary_first",
    "unary_freq",
    "arity",
    "invarity",
    "const_max",
    "const_min",
    "freq",
    "invfreq",
    "invconjfreq",
    "invfreqconjmax",
    "invfreqconjmin",
    "invfreqconstmin",
    "invfreqhack",
    "typefreq",
    "invtypefreq",
    "combfreq",
    "invcombfreq"]

weights = [
    "firstmaximal0",
    "arity",
    "aritymax0",
    "modarity",
    "modaritymax0",
    "aritysquared",
    "aritysquaredmax0",
    "invarity",
    "invaritymax0",
    "invaritysquared",
    "invaritysquaredmax0",
    "precedence",
    "invprecedence",
    "precrank5",
    "precrank10",
    "precrank20",
    "freqcount",
    "invfreqcount",
    "freqrank",
    "invfreqrank",
    "invconjfreqrank",
    "freqranksquare",
    "invfreqranksquare",
    "invmodfreqrank",
    "invmodfreqrankmax0",
    "typefreqrank",
    "typefreqcount",
    "invtypefreqrank",
    "invtypefreqcount",
    "combfreqrank",
    "combfreqcount",
    "invcombfreqrank",
    "invcombfreqcount",
    "constant"]


BS="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"

heustr_to_n = {}
i = 0
for name in clause_heuristics.keys():
    heustr_to_n[name] = i
    i = i+1


def toBase(n, b):
    return "0" if not n else toBase(n//b, b)+ BS[n%b]

def fromBase(str, b):
    """
    This will probably only ever be used for testing. If it ever is a
    bottleneck, change index() to a computation.
    """
    res = 0
    for c in str:
        res = res*b
        val = BS.index(c)
        res = res+val
    return res


def makeHenc(n):
    return ("0000"+toBase(n,36))[-4:]

def heustrToHenc(name):
    """
    Return the numerical encoding of the string name of a heuristic.
    """
    return makeHenc(heustr_to_n[name])

def makeLenc(n):
    return ("000"+to_base(n,36))[-3:]




class TestEncodinfs(unittest.TestCase):

    def testBases(self):
        for i in range(0, 300, 7):
            self.assertEqual(i, fromBase(toBase(i,36), 36))

    def testEnc(self):
        for name in clause_heuristics.keys():
            print(name, "=", heustrToHenc(name))



if __name__ == '__main__':
    unittest.main()
